#!/usr/bin/env python
"""
_GetSemiFinishedTasks_

Oracle implementation of Subscriptions.GetSemiFinishedTasks

Created on Nov 7, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Subscriptions.GetSemiFinishedTasks import GetSemiFinishedTasks as MySQLGetSemiFinishedTasks

class GetSemiFinishedTasks(MySQLGetSemiFinishedTasks):
    pass
