#!/bin/bash

export BASE_DIR=$PWD
export TEST_DIR=$BASE_DIR/wmcore_unittest
export TEST_SRC=$TEST_DIR/WMCore/src
export TEST_SRC_PYTHON=$TEST_SRC/python
export INSTALL_DIR=$BASE_DIR/unittestdeploy/wmagent
export ADMIN_DIR=$BASE_DIR
export CERT_DIR=$BASE_DIR/certs
