#!/usr/bin/env python
"""
_DeleteParentCheck_

Oracle implementation of DeleteParentCheck
"""

from WMCore.WMBS.MySQL.Files.DeleteParentCheck import DeleteParentCheck as MySQLDeleteParentCheck

class DeleteParentCheck(MySQLDeleteParentCheck):
    pass
