#!/usr/bin/env python
"""
_Delete_

Oracle implementation of Locations.Delete
"""

__all__ = []



from WMCore.WMBS.MySQL.Locations.Delete import Delete as DeleteLocationsMySQL

class Delete(DeleteLocationsMySQL):
    pass